#!/bin/bash

# Register top level shell to allow exiting script from run-function
trap "exit 1" TERM
export TOP_PID=$$

###
### Define functions
###

# Use this function to execute commands
# This first line prints the command, then executes it, checks the return code and shows an error message
function runnoexit {
    echo -e "\$ $@"
    "$@"
    local status=$?
    if [[ $status -ne 0 ]]; then
        echo -e "Command '$1' returned an error with exit code '$status'\n" >&2
    fi
    echo
    return $status
}

# Use this function to execute commands
# This first line executes the command with the runnoexit wrapper, checks the return code and exits the entire script on error
function run {
    runnoexit "$@"
    local status=$?
    if [[ $status -ne 0 ]]; then
        kill -s TERM $TOP_PID
    fi
    return $status
}

# Run a script file once, if it exists.
# After successful run the file is renamed to "filename.sh-done"
function runOnce {
    [ -f "$1" ] && source "$1" && run mv "$1" "$1-done"
}

# Usage:
#   repeatchar "*" 20
# Repeats character * 20 times
function repeatchar {
    str=$1
    num=$2
    v=$(printf "%-${num}s" "$str")
    echo "${v// /*}"
}

# Print formatted header
function printheader {
    local text=${@}
    local linelength=$(( ${#text} + 12 ))
    repeatchar "*" ${linelength}
    echo -e "***   ${text}   ***"
    repeatchar "*" ${linelength}
    echo
}

# For each argument, try if a command with that name is available by running 'executable --version',
# and return the first available as $executableName Check if executable $1 is available
# Exit with return code 1 when none is available.
# Usage:
#   findExecutableAlias composer /usr/bin/composer.phar composer.phar
function findExecutableAlias {
    for exe in "$@"
    do
        $exe --version >> /dev/null 2>&1
        if [[ $? -eq 0 ]]; then
            executablePath=`which $exe`
            echo -n "Found command $executablePath: "
            $executablePath --version
            return 0
        fi
    done
    echo -e "No executable found for $1 not found, exiting";
    return 1
}

# replaceLocalConstantsSetting <setting.name> <value>
# $1 is setting name, $2 is value
# Example usage:
#   replaceLocalConstantsSetting config.no_cache 0
function replaceLocalConstantsSetting {
    if [ $# -ne 2 ]; then
        echo "WARNING: Not exactly 2 parameters supplied, skipping..."
        return
    fi
    if [ -f "${localConstantsFile}" ]; then
        # escape the search string for the sed matching
        # https://stackoverflow.com/questions/407523/escape-a-string-for-a-sed-replace-pattern
        escaped=$(printf '%s\n' "$1" | sed -e 's/[]\/$*.^[]/\\&/g')
        # Remove the line containing this setting
        sed -i "/${escaped}/d" "${localConstantsFile}"
        # Add newline if file doesn't end with a newline and is not empty
        [[ $(tail -c1 "${localConstantsFile}") && -f "${localConstantsFile}" ]] && echo '' >> "${localConstantsFile}"
        # Append the setting with value
        echo -e "$1 = $2" >> "${localConstantsFile}"
    else
        echo -e "WARNING: file '"${localConstantsFile}"' not found, setting '$1 = $2' not added or replaced"
    fi
}
